@@ -56,8 +56,8 @@ KURL StyleBase::baseURL() const
     StyleSheet* sheet = const_cast<StyleBase*>(this)->stylesheet();
     if (!sheet)
         return KURL();
-    if (!sheet->putativeBaseURL().isNull())
-        return sheet->putativeBaseURL();
+    if (!sheet->finalURL().isNull())
+        return sheet->finalURL();
     if (sheet->parent())
         return sheet->parent()->baseURL();
     if (!sheet->ownerNode()) 
