@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 Apple Inc. All rights reserved.
+ * Copyright (C) 2007, 2008, 2009, 2010 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -66,7 +66,7 @@ class InjectedScriptHost;
 class InspectorBackend;
 class InspectorClient;
 class InspectorFrontend;
-class InspectorFrontendHost;
+class InspectorFrontendClient;
 class InspectorTimelineAgent;
 class JavaScriptCallFrame;
 class KURL;
@@ -85,6 +85,7 @@ class ConsoleMessage;
 class InspectorDatabaseResource;
 class InspectorDOMStorageResource;
 class InspectorResource;
+class InspectorWorkerResource;
 
 class InspectorController
 #if ENABLE(JAVASCRIPT_DEBUGGER)
@@ -100,6 +101,7 @@ public:
     typedef HashMap<int, RefPtr<InspectorDOMStorageResource> > DOMStorageResourcesMap;
 
     typedef enum {
+        AuditsPanel,
         CurrentPanel,
         ConsolePanel,
         ElementsPanel,
@@ -114,11 +116,9 @@ public:
     ~InspectorController();
 
     InspectorBackend* inspectorBackend() { return m_inspectorBackend.get(); }
-    InspectorFrontendHost* inspectorFrontendHost() { return m_inspectorFrontendHost.get(); }
     InjectedScriptHost* injectedScriptHost() { return m_injectedScriptHost.get(); }
 
     void inspectedPageDestroyed();
-    void pageDestroyed() { m_page = 0; }
 
     bool enabled() const;
 
@@ -134,28 +134,24 @@ public:
     void show();
     void showPanel(SpecialPanels);
     void close();
-
-    bool windowVisible();
-    void setWindowVisible(bool visible = true, bool attached = false);
+    void disconnectFrontend();
 
     void addMessageToConsole(MessageSource, MessageType, MessageLevel, ScriptCallStack*);
     void addMessageToConsole(MessageSource, MessageType, MessageLevel, const String& message, unsigned lineNumber, const String& sourceID);
     void clearConsoleMessages();
     const Vector<ConsoleMessage*>& consoleMessages() const { return m_consoleMessages; }
 
-    void attachWindow();
-    void detachWindow();
-
-    void toggleSearchForNodeInPage();
     bool searchingForNodeInPage() const { return m_searchingForNode; }
     void mouseDidMoveOverElement(const HitTestResult&, unsigned modifierFlags);
     void handleMousePressOnNode(Node*);
 
+    void setInspectorFrontendClient(PassOwnPtr<InspectorFrontendClient> client);
+    bool hasInspectorFrontendClient() const { return m_inspectorFrontendClient; }
+                                                        
     void inspectedWindowScriptObjectCleared(Frame*);
-    void windowScriptObjectAvailable();
 
-    void setFrontendProxyObject(ScriptState* state, ScriptObject webInspectorObj, ScriptObject injectedScriptObj = ScriptObject());
-    ScriptState* frontendScriptState() const { return m_frontendScriptState; }
+    bool windowVisible();
+    void setFrontend(PassOwnPtr<InspectorFrontend>);
 
     void populateScriptObjects();
     void resetScriptObjects();
@@ -185,11 +181,14 @@ public:
 
     void mainResourceFiredLoadEvent(DocumentLoader*, const KURL&);
     void mainResourceFiredDOMContentEvent(DocumentLoader*, const KURL&);
-    
+
     void didInsertDOMNode(Node*);
     void didRemoveDOMNode(Node*);
     void didModifyDOMAttr(Element*);
-                                                        
+#if ENABLE(WORKERS)
+    void didCreateWorker(long id, const String& url, bool isSharedWorker);
+    void willDestroyWorker(long id);
+#endif
     void getCookies(long callId);
 
 #if ENABLE(DATABASE)
@@ -249,19 +248,22 @@ public:
     void evaluateForTestInFrontend(long callId, const String& script);
 
     InjectedScript injectedScriptForNodeId(long id);
+    void addScriptToEvaluateOnLoad(const String& source);
+    void removeAllScriptsToEvaluateOnLoad();
+
+    static const String& inspectorStartsAttachedSettingName();
 
 private:
-    static const char* const FrontendSettingsSettingName;
+    static const String& frontendSettingsSettingName();
+
     friend class InspectorBackend;
-    friend class InspectorFrontendHost;
     friend class InjectedScriptHost;
+
+    // Following are used from InspectorBackend and internally.
+    void setSearchingForNode(bool enabled);
+
     // Following are used from InspectorBackend and internally.
-    void scriptObjectReady();
-    void moveWindowBy(float x, float y) const;
-    void setAttachedWindow(bool);
-    void setAttachedWindowHeight(unsigned height);
     void storeLastActivePanel(const String& panelName);
-    void closeWindow();
     InspectorDOMAgent* domAgent() { return m_domAgent.get(); }
     void releaseDOMAgent();
 
@@ -302,8 +304,6 @@ private:
     void pruneResources(ResourcesMap*, DocumentLoader* loaderToKeep = 0);
     void removeAllResources(ResourcesMap* map) { pruneResources(map); }
 
-    void showWindow();
-
     bool isMainResourceLoader(DocumentLoader* loader, const KURL& requestUrl);
 
     SpecialPanels specialPanelForJSName(const String& panelName);
@@ -312,10 +312,11 @@ private:
 
     Page* m_inspectedPage;
     InspectorClient* m_client;
+    OwnPtr<InspectorFrontendClient> m_inspectorFrontendClient;
+    bool m_openingFrontend;
     OwnPtr<InspectorFrontend> m_frontend;
     RefPtr<InspectorDOMAgent> m_domAgent;
     OwnPtr<InspectorTimelineAgent> m_timelineAgent;
-    Page* m_page;
     RefPtr<Node> m_nodeToFocus;
     RefPtr<InspectorResource> m_mainResource;
     ResourcesMap m_resources;
@@ -331,8 +332,6 @@ private:
 #if ENABLE(DOM_STORAGE)
     DOMStorageResourcesMap m_domStorageResources;
 #endif
-    ScriptState* m_frontendScriptState;
-    bool m_windowVisible;
     SpecialPanels m_showAfterVisible;
     RefPtr<Node> m_highlightedNode;
     unsigned m_groupLevel;
@@ -341,13 +340,13 @@ private:
     bool m_resourceTrackingEnabled;
     bool m_resourceTrackingSettingsLoaded;
     RefPtr<InspectorBackend> m_inspectorBackend;
-    RefPtr<InspectorFrontendHost> m_inspectorFrontendHost;
     RefPtr<InjectedScriptHost> m_injectedScriptHost;
 
     typedef HashMap<String, String> Settings;
     mutable Settings m_settings;
 
     Vector<pair<long, String> > m_pendingEvaluateTestCommands;
+    Vector<String> m_scriptsToEvaluateOnLoad;
 #if ENABLE(JAVASCRIPT_DEBUGGER)
     bool m_debuggerEnabled;
     bool m_attachDebuggerWhenShown;
@@ -361,6 +360,11 @@ private:
     Timer<InspectorController> m_startProfiling;
     ProfilesMap m_profiles;
 #endif
+#if ENABLE(WORKERS)
+    typedef HashMap<long, RefPtr<InspectorWorkerResource> > WorkersMap;
+
+    WorkersMap m_workers;
+#endif
 };
 
 inline void InspectorController::didInsertDOMNode(Node* node)
