@@ -256,6 +256,9 @@ public:
 
     bool isSandboxed(SandboxFlags mask) const { return m_sandboxFlags & mask; }
     SandboxFlags sandboxFlags() const { return m_sandboxFlags; }
+    // The following sandbox flags will be forced, regardless of changes to
+    // the sandbox attribute of any parent frames.
+    void setForceSandboxFlags(SandboxFlags flags) { m_forceSandboxFlags = flags; m_sandboxFlags |= flags; }
 
     // Mixed content related functions.
     static bool isMixedContent(SecurityOrigin* context, const KURL&);
@@ -280,6 +283,7 @@ public:
     void setResponseMIMEType(const String&);
     const String& responseMIMEType() const;
 
+    bool allowPlugins(ReasonForCallingAllowPlugins);
     bool containsPlugins() const;
 
     void loadDone();
@@ -522,6 +526,7 @@ private:
     bool m_suppressOpenerInNewFrame;
     
     SandboxFlags m_sandboxFlags;
+    SandboxFlags m_forceSandboxFlags;
 
 #ifndef NDEBUG
     bool m_didDispatchDidCommitLoad;
