@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 Apple Inc.  All rights reserved.
+ * Copyright (C) 2007, 2009 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,6 +41,17 @@
 
 namespace WebCore {
 
+static Node* selectionShadowAncestor(Frame* frame)
+{
+    Node* node = frame->selection()->selection().base().anchorNode();
+    if (!node)
+        return 0;
+    Node* shadowAncestor = node->shadowAncestorNode();
+    if (shadowAncestor == node)
+        return 0;
+    return shadowAncestor;
+}
+
 DOMSelection::DOMSelection(Frame* frame)
     : m_frame(frame)
 {
@@ -88,6 +99,8 @@ Node* DOMSelection::anchorNode() const
 {
     if (!m_frame)
         return 0;
+    if (Node* shadowAncestor = selectionShadowAncestor(m_frame))
+        return shadowAncestor->parentNode();
     return anchorPosition(visibleSelection()).node();
 }
 
@@ -95,6 +108,8 @@ int DOMSelection::anchorOffset() const
 {
     if (!m_frame)
         return 0;
+    if (Node* shadowAncestor = selectionShadowAncestor(m_frame))
+        return shadowAncestor->nodeIndex();
     return anchorPosition(visibleSelection()).deprecatedEditingOffset();
 }
 
@@ -102,6 +117,8 @@ Node* DOMSelection::focusNode() const
 {
     if (!m_frame)
         return 0;
+    if (Node* shadowAncestor = selectionShadowAncestor(m_frame))
+        return shadowAncestor->parentNode();
     return focusPosition(visibleSelection()).node();
 }
 
@@ -109,6 +126,8 @@ int DOMSelection::focusOffset() const
 {
     if (!m_frame)
         return 0;
+    if (Node* shadowAncestor = selectionShadowAncestor(m_frame))
+        return shadowAncestor->nodeIndex();
     return focusPosition(visibleSelection()).deprecatedEditingOffset();
 }
 
@@ -116,6 +135,8 @@ Node* DOMSelection::baseNode() const
 {
     if (!m_frame)
         return 0;
+    if (Node* shadowAncestor = selectionShadowAncestor(m_frame))
+        return shadowAncestor->parentNode();
     return basePosition(visibleSelection()).node();
 }
 
@@ -123,14 +144,17 @@ int DOMSelection::baseOffset() const
 {
     if (!m_frame)
         return 0;
+    if (Node* shadowAncestor = selectionShadowAncestor(m_frame))
+        return shadowAncestor->nodeIndex();
     return basePosition(visibleSelection()).deprecatedEditingOffset();
 }
 
-
 Node* DOMSelection::extentNode() const
 {
     if (!m_frame)
         return 0;
+    if (Node* shadowAncestor = selectionShadowAncestor(m_frame))
+        return shadowAncestor->parentNode();
     return extentPosition(visibleSelection()).node();
 }
 
@@ -138,13 +162,15 @@ int DOMSelection::extentOffset() const
 {
     if (!m_frame)
         return 0;
+    if (Node* shadowAncestor = selectionShadowAncestor(m_frame))
+        return shadowAncestor->nodeIndex();
     return extentPosition(visibleSelection()).deprecatedEditingOffset();
 }
 
 bool DOMSelection::isCollapsed() const
 {
-    if (!m_frame)
-        return false;
+    if (!m_frame || selectionShadowAncestor(m_frame))
+        return true;
     return !m_frame->selection()->isRange();
 }
 
@@ -181,6 +207,10 @@ void DOMSelection::collapse(Node* node, int offset, ExceptionCode& ec)
         ec = INDEX_SIZE_ERR;
         return;
     }
+
+    if (!isValidForPosition(node))
+        return;
+
     m_frame->selection()->moveTo(VisiblePosition(node, offset, DOWNSTREAM));
 }
 
@@ -206,7 +236,7 @@ void DOMSelection::empty()
 {
     if (!m_frame)
         return;
-    m_frame->selection()->moveTo(VisiblePosition());
+    m_frame->selection()->clear();
 }
 
 void DOMSelection::setBaseAndExtent(Node* baseNode, int baseOffset, Node* extentNode, int extentOffset, ExceptionCode& ec)
@@ -218,6 +248,10 @@ void DOMSelection::setBaseAndExtent(Node* baseNode, int baseOffset, Node* extent
         ec = INDEX_SIZE_ERR;
         return;
     }
+
+    if (!isValidForPosition(baseNode) || !isValidForPosition(extentNode))
+        return;
+
     VisiblePosition visibleBase = VisiblePosition(baseNode, baseOffset, DOWNSTREAM);
     VisiblePosition visibleExtent = VisiblePosition(extentNode, extentOffset, DOWNSTREAM);
 
@@ -232,6 +266,10 @@ void DOMSelection::setPosition(Node* node, int offset, ExceptionCode& ec)
         ec = INDEX_SIZE_ERR;
         return;
     }
+
+    if (!isValidForPosition(node))
+        return;
+
     m_frame->selection()->moveTo(VisiblePosition(node, offset, DOWNSTREAM));
 }
 
@@ -294,14 +332,20 @@ void DOMSelection::extend(Node* node, int offset, ExceptionCode& ec)
         ec = TYPE_MISMATCH_ERR;
         return;
     }
+
     if (offset < 0 || offset > (node->offsetInCharacters() ? caretMaxOffset(node) : (int)node->childNodeCount())) {
         ec = INDEX_SIZE_ERR;
         return;
     }
 
+    if (!isValidForPosition(node))
+        return;
+
     SelectionController* selection = m_frame->selection();
     selection->expandUsingGranularity(CharacterGranularity);
     selection->setExtent(VisiblePosition(node, offset, DOWNSTREAM));
+
+    m_frame->selection()->setExtent(VisiblePosition(node, offset, DOWNSTREAM));
 }
 
 PassRefPtr<Range> DOMSelection::getRangeAt(int index, ExceptionCode& ec)
@@ -317,6 +361,12 @@ PassRefPtr<Range> DOMSelection::getRangeAt(int index, ExceptionCode& ec)
     // If you're hitting this, you've added broken multi-range selection support
     ASSERT(rangeCount() == 1);
 
+    if (Node* shadowAncestor = selectionShadowAncestor(m_frame)) {
+        Node* container = shadowAncestor->parentNode();
+        int offset = shadowAncestor->nodeIndex();
+        return Range::create(shadowAncestor->document(), container, offset, container, offset);
+    }
+
     const VisibleSelection& selection = m_frame->selection()->selection();
     return selection.firstRange();
 }
@@ -397,7 +447,7 @@ bool DOMSelection::containsNode(const Node* n, bool allowPartial) const
 
     SelectionController* selection = m_frame->selection();
 
-    if (!n || selection->isNone())
+    if (!n || m_frame->document() != n->document() || selection->isNone())
         return false;
 
     Node* parentNode = n->parentNode();
@@ -440,4 +490,12 @@ String DOMSelection::toString()
     return plainText(m_frame->selection()->selection().toNormalizedRange().get());
 }
 
+bool DOMSelection::isValidForPosition(Node* node) const
+{
+    ASSERT(m_frame);
+    if (!node)
+        return true;
+    return node->document() == m_frame->document();
+}
+
 } // namespace WebCore
