@@ -25,12 +25,19 @@
  *
  */
 
-#ifndef RenderThemeGdk_h
-#define RenderThemeGdk_h
+#ifndef RenderThemeGtk_h
+#define RenderThemeGtk_h
 
+#include "GRefPtr.h"
 #include "RenderTheme.h"
 
-#include <gtk/gtk.h>
+typedef struct _GtkWidget GtkWidget;
+typedef struct _GtkStyle GtkStyle;
+typedef struct _GtkContainer GtkContainer;
+typedef struct _GdkRectangle GdkRectangle;
+typedef struct _GdkDrawable GdkDrawable;
+typedef struct _GtkBorder GtkBorder;
+typedef struct _GtkThemeParts GtkThemeParts;
 
 namespace WebCore {
 
@@ -73,9 +80,17 @@ public:
 
     virtual double caretBlinkInterval() const;
 
+    virtual void platformColorsDidChange();
+
     // System fonts.
     virtual void systemFont(int propId, FontDescription&) const;
 
+#if ENABLE(VIDEO)
+    virtual String extraMediaControlsStyleSheet();
+#endif
+
+    GtkThemeParts* partsForDrawable(GdkDrawable*) const;
+
 protected:
     virtual bool paintCheckbox(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& r);
     virtual void setCheckboxSize(RenderStyle* style) const;
@@ -106,6 +121,18 @@ protected:
     virtual void adjustSearchFieldCancelButtonStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
     virtual bool paintSearchFieldCancelButton(RenderObject*, const RenderObject::PaintInfo&, const IntRect&);
 
+    virtual void adjustSliderThumbSize(RenderObject*) const;
+
+#if ENABLE(VIDEO)
+    virtual void initMediaStyling(GtkStyle* style, bool force);
+    virtual bool paintMediaFullscreenButton(RenderObject*, const RenderObject::PaintInfo&, const IntRect&);
+    virtual bool paintMediaPlayButton(RenderObject*, const RenderObject::PaintInfo&, const IntRect&);
+    virtual bool paintMediaMuteButton(RenderObject*, const RenderObject::PaintInfo&, const IntRect&);
+    virtual bool paintMediaSeekBackButton(RenderObject*, const RenderObject::PaintInfo&, const IntRect&);
+    virtual bool paintMediaSeekForwardButton(RenderObject*, const RenderObject::PaintInfo&, const IntRect&);
+    virtual bool paintMediaSliderTrack(RenderObject*, const RenderObject::PaintInfo&, const IntRect&);
+    virtual bool paintMediaSliderThumb(RenderObject*, const RenderObject::PaintInfo&, const IntRect&);
+#endif
 
 private:
     /*
@@ -120,13 +147,32 @@ private:
      */
     GtkContainer* gtkContainer() const;
 
-private:
     mutable GtkWidget* m_gtkWindow;
     mutable GtkContainer* m_gtkContainer;
     mutable GtkWidget* m_gtkEntry;
     mutable GtkWidget* m_gtkTreeView;
+
+    mutable Color m_panelColor;
+    mutable Color m_sliderColor;
+    mutable Color m_sliderThumbColor;
+
+    const int m_mediaIconSize;
+    const int m_mediaSliderHeight;
+    const int m_mediaSliderThumbWidth;
+    const int m_mediaSliderThumbHeight;
+
+    RefPtr<Image> m_fullscreenButton;
+    RefPtr<Image> m_muteButton;
+    RefPtr<Image> m_unmuteButton;
+    RefPtr<Image> m_playButton;
+    RefPtr<Image> m_pauseButton;
+    RefPtr<Image> m_seekBackButton;
+    RefPtr<Image> m_seekForwardButton;
+    Page* m_page;
+    GRefPtr<GHashTable> m_partsTable;
+
 };
 
 }
 
-#endif
+#endif // RenderThemeGtk_h
